%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014-2015 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

function plotKitagawa_GPsurface_State (plot_time, StateDim, matfile)

    [~, ~, out] = MCDCMatFile.Load(matfile);

    figure(1); clf;
    surf(squeeze(out.FV(plot_time, StateDim, 1:31, 1:31)));
    set(gca, 'XTick', 1:5:31);
    set(gca, 'XTickLabel', {'-30', '-20', '-10', '0', '10', '20', '30'});
    set(gca, 'YTick', 1:5:31);
    set(gca, 'YTickLabel', {'-30', '-20', '-10', '0', '10', '10', '30'});
    xlim([1 31]);
    ylim([1 31]);
    xlabel('Dimension 1');
    ylabel('Dimension 2');
    zlabel('Realization of GP on grid points');

    file_name = sprintf('plots/KitagawaModelPSMC_GPrealization_State_dim%d.png', StateDim);
    saveas(gcf, file_name);

    file_name = sprintf('plots/KitagawaModelPSMC_GPrealization_State_dim%d.eps', StateDim);
    print('-r0','-depsc2', file_name);

    figure(7);clf;
    func = zeros(61, 61, 2);
    grid = zeros(61 * 61, 2);
    for k = -30:1:30
        for l = -30:1:30
            m = (31 + k) / 1;
            n = (31 + l) / 1;
            p = 61 * (m - 1) + n;
            grid(p, 1) = k;
            grid(p, 2) = l;
        end
    end
    tmp = out.FnState{plot_time}(grid);
    for k=-30:1:30
        for l=-30:1:30
            m = (31 + k) / 1;
            n = (31 + l) / 1;
            p = 61 * (m - 1) + n;
            func(m, n, 1) = tmp(p, 1);
            func(m, n, 2) = tmp(p, 2);
        end
    end
    surf(func(1:61, 1:61, StateDim));
    set(gca, 'XTick', 1:10:61);
    set(gca, 'XTickLabel', {'-30', '-20', '-10', '0', '10', '20', '30'});
    set(gca, 'YTick', 1:10:61);
    set(gca, 'YTickLabel', {'-30', '-20', '-10', '0', '10', '20', '30'});
    xlim([1 61]);
    ylim([1 61]);
    xlabel('Dimension 1');
    ylabel('Dimension 2');
    zlabel('Tensor product spline');

    file_name = sprintf('plots/KitagawaModelPSMC_GPsurface_State_dim%d.png', StateDim);
    saveas(gcf, file_name);

    file_name = sprintf('plots/KitagawaModelPSMC_GPsurface_State_dim%d.eps', StateDim);
    print('-r0','-depsc2', file_name);

end
